%top{
#define YY_NO_UNISTD_H
#include <iostream>
#include <prdtoa.h>
#define IN_LEXER
#include "TextRepScanClass.h"
#include "TextRepParse.tab.h"
}
%option c++
%option outfile="TextRepScanner.cpp"
%option prefix="TextRep"
%option noyywrap
%x stringsc

%%
<stringsc>\\\. {
  mColumn += 2;
  char c = YYText()[1];
  switch (c)
  {
  case 'n':
    mLValue->mStr += '\n';
    break;
  case 'r':
    mLValue->mStr += '\r';
    break;
  case 't':
    mLValue->mStr += '\t';
    break;
  case 'f':
    mLValue->mStr += '\f';
    break;
  default:
    mLValue->mStr += c;
  }
}
<stringsc>\" {
  mColumn++;
  BEGIN(INITIAL);
  return QUOTED;
}
<stringsc>. {
  mColumn++;
  mLValue->mStr += YYText()[0];
}

\" {
  mColumn++;
  BEGIN(stringsc);
  mLValue->mStr = "";
}

[ \t]+ mColumn += yyleng;
d\( mColumn += 2; return T_DIFF;
del\( mColumn += 4; return T_PARTIALDIFF;
piecewise mColumn += 9; return PIECEWISE;
case mColumn += 4; return CASE;
then mColumn += 4; return THEN;
else mColumn += 4; return ELSE;
== mColumn += 2; return T_EQEQ;
!= mColumn += 2; return T_NOTEQ;
\>= mColumn += 2; return T_GREATEREQUAL;
\<= mColumn += 2; return T_LESSEQUAL;
\#[a-z|A-Z|_]+[a-z|A-Z|_|0-9]* {
  mColumn += yyleng;
  mLValue->mStr = YYText() + 1;
  mLValue->mOverrideBuiltin = true;
  return IDENTIFIER;
}
[a-z|A-Z|_]+[a-z|A-Z|_|0-9]* {
  mColumn += yyleng;
  mLValue->mStr = YYText();
  mLValue->mOverrideBuiltin = false;
  return IDENTIFIER;
}
(-)?[0-9]+(\.[0-9]*)?((E|e)(\-|\+)?[0-9]+)? {
  mColumn += yyleng;
  mLValue->mNumber = PR_strtod(YYText(), nsnull);
  return NUMBER;
}

. mColumn++; return YYText()[0];

%%
int
textrep_lex(TextRepLValue* aLValue, TextRepFlexLexer* aLexer)
{
  aLexer->mLValue = aLValue;
  return aLexer->yylex();
}
